pub struct ProverState<H = StdHash, R = StdRng>{
pub duplex_sponge_state: H,
/* private fields */
}Expand description
ProverState is the prover state in the non-interactive transformation.
It provides the secret coins of the prover for zero-knowledge, and the hash function state for the verifier’s public coins.
The internal random number generator is instantiated with sha3::Shake128,
seeded via rand::rngs::StdRng.
Safety
Leaking ProverState is equivalent to leaking the prover’s private coins, and therefore zero-knowledge.
ProverState does not implement Clone or Copy to prevent accidental state-restoration attacks.
Fields
duplex_sponge_state: HThe public coins for the protocol.
Safety
Copying this object will break the soundness guarantees installed at the ProverState level.
Implementations
Sourceimpl<H, R> ProverState<H, R>
impl<H, R> ProverState<H, R>
Sourcepub const fn rng(&mut self) -> &mut ReseedableRng<R>
pub const fn rng(&mut self) -> &mut ReseedableRng<R>
Returns the reseedable RNG bound to this transcript.
Sourcepub const fn narg_string(&self) -> &[u8] ⓘ
pub const fn narg_string(&self) -> &[u8] ⓘ
Returns the current serialized NARG string.
Sourcepub fn public_message<T: Encoding<[H::U]> + ?Sized>(&mut self, message: &T)
pub fn public_message<T: Encoding<[H::U]> + ?Sized>(&mut self, message: &T)
Input a public message to the Fiat–Shamir transformation.
A public message in this context is a message that is shared among prover and verifier outside of the NARG, and is to be included in the Fiat–Shamir transformation but not in the final NARG string.
use spongefish::ProverState;
let mut prover_state = spongefish::domain_separator!("examples"; "ProverState::public_message")
.instance(&0u32)
.std_prover();
prover_state.public_message(&123u32);
assert_eq!(prover_state.narg_string(), b"");Sourcepub fn prover_message<T: Encoding<[H::U]> + NargSerialize + ?Sized>(
&mut self,
message: &T,
)
pub fn prover_message<T: Encoding<[H::U]> + NargSerialize + ?Sized>( &mut self, message: &T, )
Input a prover message of type T into the Fiat–Shamir transformation.
T must implement Encoding<[H::U]> to be encoded in the domain of the
duplex sponge, and NargSerialize to be serialized into the NARG string.
use spongefish::ProverState;
let mut prover_state = spongefish::domain_separator!("examples"; "ProverState::prover_message")
.instance(&0u32)
.std_prover();
prover_state.prover_message(&42u32);
let expected = 42u32.to_le_bytes();
assert_eq!(prover_state.narg_string(), expected.as_slice());Sourcepub fn verifier_message<T: Decoding<[H::U]>>(&mut self) -> T
pub fn verifier_message<T: Decoding<[H::U]>>(&mut self) -> T
Returns a verifier message T that is uniformly distributed.
T must implement Decoding<[H::U]>.
Sourcepub const fn transcript(&self) -> &[u8] ⓘ
👎Deprecated: Please use ProverState::narg_string instead.
pub const fn transcript(&self) -> &[u8] ⓘ
Please use ProverState::narg_string instead.
Alias for narg_string.
Sourcepub fn challenge<T: Decoding<[H::U]>>(&mut self) -> T
👎Deprecated: Please use ProverState::verifier_message instead.
pub fn challenge<T: Decoding<[H::U]>>(&mut self) -> T
Please use ProverState::verifier_message instead.
Alias for verifier_message.
Sourcepub fn public_messages<T: Encoding<[H::U]>>(&mut self, messages: &[T])
pub fn public_messages<T: Encoding<[H::U]>>(&mut self, messages: &[T])
Input to the Fiat–Shamir transformation an array of public messages.
Sourcepub fn public_messages_iter<J>(&mut self, messages: J)
pub fn public_messages_iter<J>(&mut self, messages: J)
Input to the Fiat–Shamir transformation an iterator of public messages.
Sourcepub fn prover_messages<T: Encoding<[H::U]> + NargSerialize>(
&mut self,
messages: &[T],
)
pub fn prover_messages<T: Encoding<[H::U]> + NargSerialize>( &mut self, messages: &[T], )
Absorbs a list of prover messages at once.
Sourcepub fn prover_messages_iter<J>(&mut self, messages: J)
pub fn prover_messages_iter<J>(&mut self, messages: J)
Absorbs an iterator of prover messages.